//		   
// NTSC encoder tester for HDL-XST005
//
// Copyright(c) 2002 Tatsuyuki Satoh , All rights reserved
//
// Version : beta2
//
// Important !
//
// 1. License
//
// This program is freeware for non-commercial use. 
// The user of this source must assume all legal 
// responsibility in the use of this program. 
// This is not GPL.
//
// 2. Guarantee 
//
// An author does no guarantee about this program.
// You can use this under your own risk. 
// 
// 3. A rule to distribute.
//   Do either 1) or 2).
//   1)Include a perfect original source.
//   2)Include all these headers , and add document of 
//     modification points
//
// -----------------------------------------------------
//
// Hardware design
//
// - HDL XSP005 (XC2S100 training board) , 4x7SEG,8xLED,1xBuzzer,4xPSW
// - 14.318MHz X'tal OSC in EXO-3 socket (PCLK)
// - R2R Direct-DAC circuit , 75/150ohm/8bit
//
// switch function
//
// PSW1 : Y level +
// PSW2 : Y level -
// PSW3 : show V level in 7SEG(LEFT two digits)
// PSW3+PSW1 : V level +
// PSW3+PSW2 : V level -
// PSW4 : show U level in 7SEG(LEFT two digits)
// PSW4+PSW1 : U level +
// PSW4+PSW2 : U level -
// PSW3+PSW4 : output Y only
// PSW1+PSW2+PSW3+PSW4 : reset video unit
//
// 7seg LED
//
// LEFT  2 digits : 00h or V or U level (HEX)
// RIGHT 2 digits : Y level (HEX)
//
// video output
//
//   0- 99 Vline : user YUV by PSW edit.
// 100-149 Vline : color bar with YUV direct.
// 150-    Vline : color bar with RGB to YUV , & Y gradation
//
// -----------------------------------------------------
//
// Histry:
// 2002.11.05 : T.satoh : beta2 , change I/F,adjust,etc.
// 2002.10.30 : T.satoh : beta1 , 1st release
// 2002. 9. 3 : T.satoh : Start Project
//
module top(
//XST005 generic assign
	PSW,LED,XSG,SA,BUZZ,PCLK,
//VOUT
	V_OUT,
	TRG
	);

//I/O assign
input [3:0] PSW;
inout [7:0] LED,XSG;
inout [3:0] SA;
output BUZZ;
input PCLK; // 14.318MHz

//VOUT
output [9:0] V_OUT;
output TRG;

//////////////////////////////////////////////////////////////////////////////

//
// 7segment dynamic LED driver I/F
//
wire [3:0] dot;
wire [1:0] led_com;
wire led_en;
wire [3:0] led_cas;
wire [15:0] led_num;

hex4_led seg_led(.HEX4(led_num),.DOT(dot),.CNT(led_com),.EN(led_en),.SEG(XSG),.COM(led_cas) );

assign SA[3] = led_cas[0] ? 1'bz : 1'b0;
assign SA[2] = led_cas[1] ? 1'bz : 1'b0;
assign SA[1] = led_cas[2] ? 1'bz : 1'b0;
assign SA[0] = led_cas[3] ? 1'bz : 1'b0;

//
// tick counter (PCLK = 14.318MHz)
// ticker[16] = @140Hz
// ticker[24] = @1Hz
//
reg  [43:0] ticker;

always @(posedge PCLK)
begin
  ticker <= ticker + 1;
end

//
// RESET signal
//
wire video_reset = ~PSW[0]&~PSW[1]&~PSW[2]&~PSW[3];

//
// RGB to YUV color space converter
//
reg  [7:0] R,G,B;
wire [7:0] CY,CU,CV;
rgb_yuv  rgb_yuv1 (.R(R),.G(G),.B(B),.Y(CY),.U(CU),.V(CV));

//
// NTSC encoder I/F
//
wire [7:0] cout,yout,vout;
wire [9:0] hcnt;
wire [8:0] vcnt;
wire [1:0] sc;
wire vs , hs , cs , ColorBurst , Window , field;
reg [7:0] tpy,tpv,tpu;
reg [7:0] usy,usv,usu;

ntsc_enc test_pattern (
	.T4SC_I(PCLK),
	.RESET(video_reset),
	.Y(tpy) , .V(tpv) , .U(tpu),
	.HS_I(1'b0), .VS_I(1'b0),
//output
	.VPOS_O(vcnt),
	.HPOS_O(hcnt),
	.FIELD(field),
	.HSYNC_O(hs),
	.VSYNC_O(vs),
	.CSYNC_O(cs),
	.Y_O(yout),
	.C_O(cout),
	.V_O(vout),
//debug
	.SC_O(sc),
	.CB_O(ColorBurst),
	.WIN_O(Window)
);

//
// TestPattern Generator
//
parameter BAR_B  = 10;  // left space 
parameter BAR_W  = 85;  // BAR width 
parameter USER_V = 50;  // V-END user area
parameter YUV_V  = 75;  // V-END YUV BAR
parameter RGB_V  = 100; // V-END RGB BAR1

// RGB fade level
reg [7:0] RGB_H , RGB_L;

// V fader
reg v_usr , v_rgb , v_yuv;
always @(vcnt)
begin
	RGB_H <= (vcnt>=RGB_V) ? ((vcnt-RGB_V)<<1) : 8'hff;

	v_usr = vcnt < USER_V;
	v_rgb = (vcnt>=RGB_V) & PSW[2];
	v_yuv = vcnt>=YUV_V;
end

always @(posedge PCLK)
begin
	// RGB L level
	if(~v_rgb)
		RGB_L <= 0;
	else begin
		if((hcnt==BAR_B) || (RGB_L==8'hff))
			// reset counter
			RGB_L <= 8'hff-((BAR_W-1)*3);
		else begin
			RGB_L <= RGB_L + 3;
		end
	end

	if(v_usr)
	begin // USER set
		tpy <= usy;
		tpv <= usv;
		tpu <= usu;
//	end else if(field & vcnt[0])
//	begin
//		tpy <= 9'h00;
//		tpv <= 9'h00;
//		tpu <= 9'h00;
	end else
	begin
		if(hcnt < (BAR_B+1*BAR_W))
		begin  // K
			R <= RGB_L;
			G <= RGB_L;
			B <= RGB_L;
			tpy <= 0;
			tpv <= 0;
			tpu <= 0;
		end else if(hcnt < (BAR_B+2*BAR_W))
		begin // BL
			R <= RGB_L;
			G <= RGB_L;
			B <= RGB_H;
			tpy <= 12;
			tpv <= -11;
			tpu <= 49;
		end else if(hcnt < (BAR_B+3*BAR_W))
		begin // RD
			R <= RGB_H;
			G <= RGB_L;
			B <= RGB_L;
			tpy <= 33;
			tpv <= 69;
			tpu <= -16;
		end else if(hcnt < (BAR_B+4*BAR_W))
		begin // MA
			R <= RGB_H;
			G <= RGB_L;
			B <= RGB_H;
			tpy <= 46;
			tpv <= 58;
			tpu <= 32;
		end else if(hcnt < (BAR_B+5*BAR_W))
		begin // GR
			R <= RGB_L;
			G <= RGB_H;
			B <= RGB_L;
			tpy <= 66;
			tpv <= -58;
			tpu <= -32;
		end else if(hcnt < (BAR_B+6*BAR_W))
		begin // CY
			R <= RGB_L;
			G <= RGB_H;
			B <= RGB_H;
			tpy <= 79;
			tpv <= -69;
			tpu <= 16;
		end else if(hcnt < (BAR_B+7*BAR_W))
		begin // YE
			R <= RGB_H;
			G <= RGB_H;
			B <= RGB_L;
			tpy <= 100;
			tpv <= 11;
			tpu <= -49;
		end else if(hcnt < (BAR_B+8*BAR_W))
		begin // WH
			R <= RGB_H;
			G <= RGB_H;
			B <= RGB_H;
			tpy <= 113;
			tpv <= 0;
			tpu <= 0;
		end else if(hcnt < (BAR_B+9*BAR_W))
		begin  // K
			R <= RGB_L;
			G <= RGB_L;
			B <= RGB_L;
			tpy <= 9'h00;
			tpv <= 9'h00;
			tpu <= 9'h00;
		end

		// RGB or YUV
		if(v_yuv)
		begin // RGB to YUV colorbar
			tpy <= CY;
			tpv <= CV;
			tpu <= CU;
		end
	end
end

//
// composite Video DAC
//
// 9bit fullscale -> 8bit 75ohm/0.75Vpp
// 500/1K/2K/4K/8K/16K/32K/64K
// conversion table
wire [7:0] vot;
//RAMB4_S8 vtable(.CLK(t4sc),.ADDR(vout),.DO(vot),.DI(8'h00),.EN(1'b1),.WE(1'b0),.RST(1'b0));

assign vot =  PSW[3] ? vout[7:0] : yout[7:0];
//assign vot = ~PSW[1] ? cout[7:0] : ~PSW[0] ? yout[7:0] : vout[7:0];
//assign vot = hcnt[9:2];
// I/O pin assign
OBUF_S_24 V0(.O(V_OUT[0]),.I(vot[7]));
OBUF_S_24 V1(.O(V_OUT[1]),.I(vot[6]));
OBUF_S_24 V2(.O(V_OUT[2]),.I(vot[5]));
OBUF_S_24 V3(.O(V_OUT[3]),.I(vot[4]));
OBUF_S_24 V4(.O(V_OUT[4]),.I(vot[3]));
OBUF_S_24 V5(.O(V_OUT[5]),.I(vot[2]));
OBUF_S_24 V6(.O(V_OUT[6]),.I(vot[1]));
OBUF_S_24 V7(.O(V_OUT[7]),.I(vot[0]));
OBUF_S_24 V8(.O(V_OUT[8]),.I(1'b0));
OBUF_S_24 V9(.O(V_OUT[9]),.I(1'b0));

//
// test trigger
//

//assign TRG = hs;
assign TRG = ColorBurst;

//
// user I/F
//
// PSW1 = up   value
// PSW2 = down value
// PSW3 = cr select
// PSW4 = cb select


reg [3:0] repeat_cnt;
always @(posedge ticker[18])
begin
	// repeat counter
	if(PSW[1]&PSW[0])
		repeat_cnt <= 0;
	else if(repeat_cnt != 4'hf)
		repeat_cnt <= repeat_cnt + 1;
 
	//up / down
	if( (repeat_cnt==0) || (repeat_cnt==4'hf) )
 	begin
		if(~PSW[0])			 
		begin
			if(~PSW[3])
				usu <= usu + 1;
			else if(~PSW[2])
				usv <= usv + 1;
			else
				usy <= usy + 1;
		end
		if(~PSW[1])			 
		begin
			if(~PSW[3])
				usu <= usu - 1;
			else if(~PSW[2])
				usv <= usv - 1;
			else
				usy <= usy - 1;
		end
	end 
end

//
// assign 7segment led : 
//
//
assign led_num = ~PSW[3] ? { usu,usy }  :
				 ~PSW[2] ? { usv,usy }  :
						   { 8'h00,usy };

assign dot[0] = sc[1:0]==2'b00 ? 1'b1 : 1'b0; // sc0
assign dot[1] = sc[1:0]==2'b01 ? 1'b1 : 1'b0; // sc1
assign dot[2] = sc[1:0]==2'b10 ? 1'b1 : 1'b0; // sc2
assign dot[3] = sc[1:0]==2'b11 ? 1'b1 : 1'b0; // sc3
assign led_com = ticker[16:15]; // 140Hz cycle
assign led_en  = !(ticker[14:0] == 0);

//
// static LED x8 driver
//
assign LED[0] = PCLK;
assign LED[1] = 1'b0;
assign LED[2] = vs   ? 1'b0 : 1'bz;
assign LED[3] = hs   ? 1'b0 : 1'bz;
assign LED[4] = ColorBurst ? 1'b0 : 1'bz;
assign LED[5] = Window     ? 1'b0 : 1'bz;
assign LED[6] = PSW[2] ? 1'b0 : 1'bz;
assign LED[7] = PSW[3] ? 1'b0 : 1'bz;

//
// BUZZER driver
//
assign BUZZ = 1'bz;

endmodule

